#######################################################################################################################

# -- Example Code for using the Variational Inference for Computational Imaging (VICI) Model --

#######################################################################################################################

import numpy as np
import tensorflow as tf
import scipy.io as sio
import scipy.misc as mis
import h5py
from sys import exit
from Models import VICI_inverse_model
from Models import CVAE
from Neural_Networks import batch_manager

#from Observation_Models import simulate_observations
from data import make_samples, chris_data
import plots

run_label='wiay',                                         # label for run
plot_dir="/data/public_html/chrism/VICI/%s" % run_label,  # plot directory
ndata=128                                                 # y dimension size
load_mcmc_samp = True        # if True, load previously made train/test samples (including mcmc run).
do_grid = False               # if True use gridded likelihood values
run_inverse = True            # if True run inverse network

# Defining the list of parameter that need to be fed into the models
def get_params():
    params = dict(
        image_size = [1,ndata], # Images Size
        print_values=True, # optionally print values every report interval
        n_samples = 5000, # number of posterior samples to save per reconstruction upon inference
        #------------------------------------------------------------------------- 
        num_iterations=2000000, # number of iterations inference model (inverse reconstruction)
        initial_training_rate=0.00001, # initial training rate for ADAM optimiser inference model (inverse reconstruction)
        batch_size=128, # batch size inference model (inverse reconstruction)
        report_interval=500, # interval at which to save objective function values and optionally print info during inference training
        z_dimension=48, # number of latent space dimensions inference model (inverse reconstruction)
        n_weights = 2048, # number of dimensions of the intermediate layers of encoders and decoders in the inference model (inverse reconstruction)
        save_interval=10000, # interval at which to save inference model weights
        #-------------------------------------------------------------------------
        plot_interval=10000,
        newdata_interval=int(2**13),
        newdata_size=int(2**20),
        ndata = ndata,
        r =5,                      # the grid dimension for the output test
        sigma=0.2,                   # stadnard deviation of the noise on signal
        usepars=[0,1,2],             # which parameters you want to do PE on
        varpars=[0,1,2,3],           # which pars do you want to vary
        ngrid=64j,                    # number of grid points pre dimension for likelihood map
        tot_dataset_size=int(2**20), # total size of training set
        seed=42,                     # random seed number
        run_label=run_label,            # label for run
        plot_dir=plot_dir,                 # plot directory
        parnames=['A','t0','tau','phi','w'],    # parameter names
        wrap=[False,False,False,True,False]    # do we wrap the parameter
    )
    return params 

# Get the training/test data and parameters of run
params=get_params()

# Get mcmc samples and test data
if load_mcmc_samp:
    hf = h5py.File('data/generated_samples_%s' % run_label, 'r')
    y_test = np.array(hf.get('y_test') )
    x_test = np.array(hf.get('x_test'))
    samples = np.array(hf.get('samples'))
    likgrid = None
    if do_grid:
        likgrid = np.array(hf.get('likgrid'))
    hf.close()
else:
    x_test,y_test = make_samples.get_sets(params,params['r']*params['r'])
    samples = chris_data.mcmc_sampler(params['r'],params['n_samples'],y_test,params['sigma'],params['usepars'],params['varpars'])
    likgrid = None
    if do_grid:
        likgrid = chris_data.grid_sampler(params['r'],y_test,params['sigma'],params['usepars'],params['varpars'],params['ngrid'])

    # save samples and test data for later
    f = h5py.File("data/generated_samples_%s" % run_label, "w")
    f.create_dataset("y_test", data=y_test)
    f.create_dataset("x_test", data=x_test)
    f.create_dataset("samples", data=samples)
    if do_grid:
        f.create_dataset("likgrid", data=likgrid)
    f.close()

# Make directory for plots
plots.make_dirs(params['plot_dir'][0])

# Declare plot class variables
plotter = plots.make_plots(params,samples,likgrid,None,x_test)

# We then train the inference model using all training images and associated observations.
if run_inverse:

    # This trains the inverse model to recover posteriors and saves the
    # inverse model weights in inverse_model_dir/inverse_model.ckpt
    loss_inv, kl_inv = VICI_inverse_model.train(params,
                                                "inverse_model_dir/inverse_model.ckpt",
                                                plotter,
                                                y_test,
                                                x_test,
                                                samples,
                                                likgrid) 


